@@ -382,6 +382,7 @@ static int plugin_run_or_verify(
     gss_cred_id_t       cred        = GSS_C_NO_CREDENTIAL;
     X509 *              px509_cred  = NULL;
     STACK_OF(X509) *    px509_chain = NULL;
+    void *              value       = NULL;
 
 #ifdef TESTBIO
     int err;
@@ -409,14 +410,18 @@ static int plugin_run_or_verify(
     /*
      * Try to get the ordered values:
      */
-    if ( ( dn = *(char **) lcmaps_getArgValue("user_dn", "char *", argc, argv) ) )
+    if ( ( value = lcmaps_getArgValue("user_dn", "char *", argc, argv) ) )  {
+	dn= *(char **)value;
         lcmaps_log_debug(1,"%s: found dn: %s\n", logstr,dn);
-    else
+    } else {
+	dn = NULL;
         lcmaps_log_debug(1,"%s: could not get value of dn !\n", logstr);
+    }
 
     /* Fetch user gss credential */
-    if ( ( pcred = (gss_cred_id_t *) lcmaps_getArgValue("user_cred", "gss_cred_id_t", argc, argv) ) )
+    if ( ( value = lcmaps_getArgValue("user_cred", "gss_cred_id_t", argc, argv) ) )
     {
+	pcred = (gss_cred_id_t *) value;
         lcmaps_log_debug(2,"%s: address user_cred: %p\n", logstr,pcred);
         cred = *pcred;
         if (cred == GSS_C_NO_CREDENTIAL)
@@ -426,7 +431,8 @@ static int plugin_run_or_verify(
         }
     }
     else
-    {
+    {	
+	pcred = NULL;
         lcmaps_log(LOG_DEBUG,"%s: could not get address of user_cred (exit voms)!\n", logstr);
         goto fail_voms;
     }
