@@ -128,6 +128,10 @@
 /*****************************************************************************
                             Include header files
 ******************************************************************************/
+/* Try to provide RTLD_DEFAULT */
+#define _GNU_SOURCE
+#include <dlfcn.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -295,9 +299,49 @@ int plugin_introspect(
         {"user_dn"      ,       "char *"        , 0,   NULL},
         {"fqan_list"    ,       "char **"       , 0,   NULL},
         {"nfqan"        ,       "int"           , 0,   NULL},
+        {NULL           ,       NULL            , -1,   NULL},
         {NULL           ,       NULL            , -1,   NULL}
     };
 
+    /* Get the version of LCMAPS being used: we need at least 1.5.8 to be able
+     * to demand "requested_username" in the argList */
+    int major=0,minor=0,patch=0;
+    /* Most UNIX now support RTLD_DEFAULT (POSIX reserved) */
+#ifdef RTLD_DEFAULT
+    char *errorstr;
+    int (*lcmaps_major)(void),(*lcmaps_minor)(void),(*lcmaps_patch)(void);
+    dlerror();
+    lcmaps_major=dlsym(RTLD_DEFAULT,"lcmaps_get_major_version");
+    lcmaps_minor=dlsym(RTLD_DEFAULT,"lcmaps_get_minor_version");
+    lcmaps_patch=dlsym(RTLD_DEFAULT,"lcmaps_get_patch_version");
+    if (dlerror()==NULL)    {
+	major=lcmaps_major();
+	minor=lcmaps_minor();
+	patch=lcmaps_patch();
+    }
+#else
+#warning old LCMAPS versions do not have lcmaps_get_major_version() functions
+    /* No RTLD_DEFAULT, just hope the symbol exists in LCMAPS */
+    major=lcmaps_get_major_version();
+    minor=lcmaps_get_minor_version();
+    patch=lcmaps_get_patch_version();
+#endif
+
+    /* Too old when older than 1.5.8 */
+    if (major<1 || (major==1 && (minor<5 || (minor==5 && patch<8))))	{
+	lcmaps_log_debug(4,
+	    "%s: Old LCMAPS found (%d.%d.%d), not using requested_username\n",
+	    logstr,major,minor,patch);
+    } else {
+	lcmaps_log_debug(4,
+	    "%s LCMAPS (%d.%d.%d) supports using requested_username\n",
+	    logstr,major,minor,patch);
+	argList[3].argName="requested_username";
+	argList[3].argType="char *";
+	argList[3].argInOut=1;
+	argList[3].value=NULL;
+    }
+
     lcmaps_log_debug(4,"%s: introspecting\n", logstr);
 
     *argv = argList;
@@ -373,6 +417,9 @@ static int plugin_run_or_verify(
     int                 rc                  = 0;
     char **             fqan_list           = NULL;
     int                 nfqan               = -1;
+    char *              req_username        = NULL;
+    char *              searchstr           = NULL;
+    void *              value               = NULL;
 
     /*
      * The beginning
@@ -391,8 +438,9 @@ static int plugin_run_or_verify(
     /*
      * Try to get the ordered values:
      */
-    if ( ( dn = *(char **) lcmaps_getArgValue("user_dn", "char *", argc, argv) ) )
+    if ( (value = lcmaps_getArgValue("user_dn", "char *", argc, argv) ) )
     {
+	dn = *(char **)value; 
         lcmaps_log_debug(5,"%s: found dn: %s\n", logstr, dn);
 
         /* Check if we don't have a DN already registered, if not, add it to the internal registry */
@@ -403,9 +451,21 @@ static int plugin_run_or_verify(
             addCredentialData(DN, &dn);
         }
     }
-    else
+    else    {
+	dn = NULL;
         lcmaps_log_debug(1,"%s: could not get value of dn !\n", logstr);
+    }
 
+    /*
+     * See if we have a requested req_username: make sure not to get value
+     * immediately or we'll get a segfault
+     */
+    if ( (value=lcmaps_getArgValue("requested_username", "char *", argc, argv))!=NULL )
+	req_username=*(char **)value;
+    else
+	req_username=NULL;
+    lcmaps_log_debug(1,"%s: requested username is %s\n", logstr,
+	    req_username ? req_username : "unset");
 
     /*
      * Check the gridmapfile
@@ -430,13 +490,16 @@ static int plugin_run_or_verify(
      * The VOMS info has to matched against the info in the gridmapfile
      */
     lcmaps_log_debug(5,"%s: First try to get the FQAN list from input credential repository ...\n", logstr);
-    if ( ( nfqan = *(int *) lcmaps_getArgValue("nfqan", "int", argc, argv) ) )
+    if ( ( value = lcmaps_getArgValue("nfqan", "int", argc, argv) ) )
     {
+	nfqan = *(int *)value;
         lcmaps_log_debug(5,"%s: the list of FQANs should contain %d elements\n", logstr, nfqan);
-        if ( ( fqan_list = *(char ***) lcmaps_getArgValue("fqan_list", "char **", argc, argv) ) )
+        if ( ( value = lcmaps_getArgValue("fqan_list", "char **", argc, argv)) )   {
+	    fqan_list = *(char ***)value;
             lcmaps_log_debug(5, "%s: found list of FQANs\n", logstr);
-        else
-        {
+	}
+	else
+	{
             lcmaps_log_debug(1, "%s: could not retrieve list of FQANs!\n", logstr);
             goto fail_voms_localaccount;
         }
@@ -473,14 +536,21 @@ static int plugin_run_or_verify(
      */
     found = 0;
 
-    matching_type = MATCH_EXCLUDE|MATCH_WILD_CHARS;
+    if ( req_username )	{
+	matching_type = MATCH_EXACT|MATCH_WILD_CHARS;
+	searchstr=req_username;
+    } else {
+	matching_type = MATCH_EXCLUDE|MATCH_WILD_CHARS;
+	searchstr=".";
+    }
+
 
     for (i = 0; i < cnt_vo_cred_string; i++)
     {
         /* clean username before each call to lcmaps_gridlist */
         if (username) free(username);
         username = NULL;
-        if ( (rc = lcmaps_gridlist(vo_cred_string_list[i], &username, gridmapfile, matching_type, ".", NULL)) == 0)
+        if ( (rc = lcmaps_gridlist(vo_cred_string_list[i], &username, gridmapfile, matching_type, searchstr, NULL)) == 0)
         {
             found = 1;
             lcmaps_log_debug(3,"%s: found username: %s\n", logstr, username);
@@ -493,7 +563,10 @@ static int plugin_run_or_verify(
         }
         else
         {
-            lcmaps_log_debug(1, "%s: no localaccount available for group (%s) in %s\n", logstr, vo_cred_string_list[i], gridmapfile);
+	    if (req_username)
+		lcmaps_log_debug(1, "%s: no localaccount available for group (%s) and requested user %s in %s\n", logstr, vo_cred_string_list[i], req_username, gridmapfile);
+	    else
+		lcmaps_log_debug(1, "%s: no localaccount available for group (%s) in %s\n", logstr, vo_cred_string_list[i], gridmapfile);
         }
     }
     if (found != 1)
