@@ -354,6 +354,7 @@ static int plugin_run_or_verify(
     lcmaps_vo_mapping_t * lcmaps_vo_mapping   = NULL;
     char **               fqan_list           = NULL;
     int                   nfqan               = -1;
+    void *                value               = NULL;
 
     /*
      * The beginning
@@ -372,8 +373,9 @@ static int plugin_run_or_verify(
     /*
      * Try to get the ordered values:
      */
-    if ( (dn = *(char **) lcmaps_getArgValue("user_dn", "char *", argc, argv) ) )
+    if ( (value = lcmaps_getArgValue("user_dn", "char *", argc, argv) ) )
     {
+	dn = *(char **)value; 
         lcmaps_log_debug(5,"%s: found dn: %s\n", logstr, dn);
 
         /* Check if we don't have a DN already registered, if not, add it to the internal registry */
@@ -384,8 +386,10 @@ static int plugin_run_or_verify(
             addCredentialData(DN, &dn);
         }
     }
-    else
+    else    {
+	dn = NULL;
         lcmaps_log_debug(1,"%s: could not get value of dn !\n", logstr);
+    }
 
     /*
      * Get the VO user information.
@@ -397,12 +401,15 @@ static int plugin_run_or_verify(
      * The VOMS info has to matched against the info in the gridmapfile
      */
     lcmaps_log_debug(1,"%s: First try to get the FQAN list from input credential repository ...\n", logstr);
-    if ( ( nfqan = *(int *) lcmaps_getArgValue("nfqan", "int", argc, argv) ) )
+    if ( ( value = lcmaps_getArgValue("nfqan", "int", argc, argv) ) )
     {
+	nfqan = *(int *) value;
         lcmaps_log_debug(1,"%s: the list of FQANs should contain %d elements\n", logstr, nfqan);
-        if ( ( fqan_list = *(char ***) lcmaps_getArgValue("fqan_list", "char **", argc, argv) ) )
+        if ( ( value = lcmaps_getArgValue("fqan_list", "char **", argc, argv) ) )   {
+	    fqan_list = *(char ***) value;
             lcmaps_log_debug(1, "%s: found list of FQANs\n", logstr);
-        else
+	}
+	else
         {
             lcmaps_log_debug(1, "%s: could not retrieve list of FQANs (failure)!\n", logstr);
             goto fail_voms_localgroup;
