@@ -190,6 +190,10 @@
 /*****************************************************************************
                             Include header files
 ******************************************************************************/
+/* Try to provide RTLD_DEFAULT */
+#define _GNU_SOURCE
+#include <dlfcn.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -441,9 +445,49 @@ int plugin_introspect(
         {"nfqan"            ,   "int"       ,   0   ,   NULL},
         {"mapcounter"       ,   "int"       ,   1   ,   NULL},
         {"requested_uid"    ,   "uid_t"     ,   0   ,   NULL},
+        {NULL               ,   NULL        ,   -1  ,   NULL},
         {NULL               ,   NULL        ,   -1  ,   NULL}
     };
 
+    /* Get the version of LCMAPS being used: we need at least 1.5.8 to be able
+     * to demand "requested_username" in the argList */
+    int major=0,minor=0,patch=0;
+    /* Most UNIX now support RTLD_DEFAULT (POSIX reserved) */
+#ifdef RTLD_DEFAULT
+    char *errorstr;
+    int (*lcmaps_major)(void),(*lcmaps_minor)(void),(*lcmaps_patch)(void);
+    dlerror();
+    lcmaps_major=dlsym(RTLD_DEFAULT,"lcmaps_get_major_version");
+    lcmaps_minor=dlsym(RTLD_DEFAULT,"lcmaps_get_minor_version");
+    lcmaps_patch=dlsym(RTLD_DEFAULT,"lcmaps_get_patch_version");
+    if (dlerror()==NULL)    {
+	major=lcmaps_major();
+	minor=lcmaps_minor();
+	patch=lcmaps_patch();
+    }
+#else
+#warning old LCMAPS versions do not have lcmaps_get_major_version() functions
+    /* No RTLD_DEFAULT, just hope the symbol exists in LCMAPS */
+    major=lcmaps_get_major_version();
+    minor=lcmaps_get_minor_version();
+    patch=lcmaps_get_patch_version();
+#endif
+
+    /* Too old when older than 1.5.8 */
+    if (major<1 || (major==1 && (minor<5 || (minor==5 && patch<8))))	{
+	lcmaps_log_debug(4,
+	    "%s: Old LCMAPS found (%d.%d.%d), not using requested_username\n",
+	    logstr,major,minor,patch);
+    } else {
+	lcmaps_log_debug(4,
+	    "%s LCMAPS (%d.%d.%d) supports using requested_username\n",
+	    logstr,major,minor,patch);
+	argList[5].argName="requested_username";
+	argList[5].argType="char *";
+	argList[5].argInOut=1;
+	argList[5].value=NULL;
+    }    
+
     lcmaps_log_debug(5,"%s: introspecting\n", logstr);
 
     *argv = argList;
@@ -657,6 +701,7 @@ static int plugin_run_or_verify(
     char *              requested_username  = NULL;
     uid_t               requested_uid       = -1;
     struct passwd       *requested_user_info = NULL;
+    void *              value               = NULL;
 
     /*
      * The beginning
@@ -675,8 +720,9 @@ static int plugin_run_or_verify(
     /*
      * Try to get the ordered values:
      */
-    if ( ( dn = *(char **) lcmaps_getArgValue("user_dn", "char *", argc, argv) ) )
+    if ( ( value = lcmaps_getArgValue("user_dn", "char *", argc, argv) ) )
     {
+	dn = *(char **) value;
         lcmaps_log_debug(5,"%s: found dn: %s\n", logstr, dn);
 
         /* Check if we don't have a DN already registered, if not, add it to the internal registry */
@@ -690,8 +736,9 @@ static int plugin_run_or_verify(
     else
         lcmaps_log_debug(1,"%s: could not get value of dn !\n", logstr);
 
-    if ( ( pmapcounter = (int *) lcmaps_getArgValue("mapcounter", "int", argc, argv) ) )
+    if ( ( value = lcmaps_getArgValue("mapcounter", "int", argc, argv) ) )
     {
+	pmapcounter = (int *) value;
         lcmaps_log_debug(3,"%s: mapcounter: %d\n", logstr, *pmapcounter);
         mapcounter = *pmapcounter;
     }
@@ -707,13 +754,20 @@ static int plugin_run_or_verify(
      */
     if (lcmaps_mode == PLUGIN_VERIFY)
     {
-        if ( (requested_uid = *(uid_t *) lcmaps_getArgValue("requested_uid", "uid_t", argc, argv)) > 0 )
+        if ( (value = lcmaps_getArgValue("requested_uid", "uid_t", argc, argv)) > 0 )
         {
             struct passwd * requested_user = NULL;
+	    requested_uid = *(uid_t *) value;
             if ((requested_user = getpwuid(requested_uid)) != NULL)
             {
                 requested_username = strdup(requested_user->pw_name);
-                lcmaps_log_debug(3,"%s: the requested user is %s(%d)\n", logstr, requested_username, (int) requested_uid);
+		if (requested_username)
+		    lcmaps_log_debug(3,"%s: the requested user is %s(%d)\n",
+			    logstr, requested_username, (int) requested_uid);
+		else    {
+		    lcmaps_log(LOG_ERR,"%s: out of memory\n",logstr);
+		    goto fail_voms_poolaccount;
+		}
             }
             else
             {
@@ -731,6 +785,21 @@ static int plugin_run_or_verify(
             lcmaps_log(LOG_ERR,"%s: illegal request for uid < 0 (failure)\n", logstr);
             goto fail_voms_poolaccount;
         }
+    } else {
+	/*
+	 * See if we have a requested username: make sure not to get value
+	 * immediately or we'll get a segfault
+	 */
+	requested_username=NULL;
+	if ( (value = lcmaps_getArgValue("requested_username", "char *", argc, argv))!=NULL &&
+	     (*(char **)value)!=NULL &&
+	     (requested_username=strdup(*(char **)value))==NULL)
+	{
+	    lcmaps_log(LOG_ERR,"%s: out of memory\n",logstr);
+	    goto fail_voms_poolaccount;
+	}
+	lcmaps_log_debug(1,"%s: requested username is %s\n", logstr,
+		requested_username ? requested_username : "unset");
     }
 
     /*
@@ -743,11 +812,14 @@ static int plugin_run_or_verify(
      * The VOMS info has to matched against the info in the gridmapfile
      */
     lcmaps_log_debug(1,"%s: First try to get the FQAN list from input credential repository ...\n", logstr);
-    if ( ( nfqan = *(int *) lcmaps_getArgValue("nfqan", "int", argc, argv) ) )
+    if ( ( value = lcmaps_getArgValue("nfqan", "int", argc, argv) ) )
     {
+	nfqan = *(int *) value;
         lcmaps_log_debug(4,"%s: the list of FQANs should contain %d elements\n", logstr, nfqan);
-        if ( ( fqan_list = *(char ***) lcmaps_getArgValue("fqan_list", "char **", argc, argv) ) )
+        if ( ( value = lcmaps_getArgValue("fqan_list", "char **", argc, argv) ) )   {
+	    fqan_list = *(char ***) value;
             lcmaps_log_debug(4, "%s: found list of FQANs\n", logstr);
+	}
         else
         {
             lcmaps_log_debug(1, "%s: could not retrieve list of FQANs (failure)!\n", logstr);
@@ -988,8 +1060,8 @@ static int plugin_run_or_verify(
 
                 if ( (rc = lcmaps_gridlist(vo_cred_string_list[i], &username, gridmapfile, matching_type, ".", newleasename)) == 0)
                 {
-                    lcmaps_log_debug(3,"%s: found a username: %s\n", logstr, username);
-                    if (strncmp(username, requested_username, strlen(requested_username)) == 0)
+		    /* in verify mode, requested_username!=NULL */
+		    if ( strcmp(username, requested_username) == 0)
                     {
                         lcmaps_log_debug(3,"%s: found requested username: %s\n", logstr, username);
                         found_mapping = 1;
@@ -999,6 +1071,7 @@ static int plugin_run_or_verify(
                          */
                         break;
                     }
+		    lcmaps_log_debug(3,"%s: found username %s but does not match requested username %s\n", logstr, username, requested_username);
                 }
                 else if (rc == LCMAPS_MOD_NOFILE)
                 {
@@ -1026,12 +1099,15 @@ static int plugin_run_or_verify(
                 }
                 if ( (rc = lcmaps_gridlist(vo_cred_string_list[i], &username, gridmapfile, matching_type, ".", newleasename)) == 0)
                 {
-                    lcmaps_log_debug(3,"%s: found a username: %s\n", logstr, username);
-                    if (strncmp(username, requested_username, strlen(requested_username)) == 0)
-                    {
+		    /* in verify mode, requested_username!=NULL */
+		    if ( strcmp(username, requested_username) == 0)
+		    {
                         lcmaps_log_debug(3,"%s: found requested username: %s\n", logstr, username);
                         found_mapping = 1;
                     }
+		    else
+                        lcmaps_log_debug(3,"%s: found username %s but does not match requested username: %s\n", logstr, username, requested_username);
+
                 }
                 else if (rc == LCMAPS_MOD_NOFILE)
                 {
@@ -1065,7 +1141,8 @@ static int plugin_run_or_verify(
                 if (lcmaps_mode == PLUGIN_VERIFY)
                 {
                     lcmaps_log_debug(3,"%s: found a username: %s\n", logstr, username);
-                    if (strncmp(username, requested_username, strlen(requested_username)) == 0)
+		    /* in verify mode, requested_username!=NULL */
+                    if (strcmp(username, requested_username) == 0)
                     {
                         lcmaps_log_debug(3,"%s: found requested username: %s\n", logstr, username);
                         found_mapping = 1;
@@ -1079,8 +1156,22 @@ static int plugin_run_or_verify(
                 }
                 else
                 {
-                    found_mapping = 1;
-                    lcmaps_log_debug(3,"%s: found username: %s\n", logstr, username);
+		    /* requested_username can be NULL */
+		    if (requested_username==NULL)   {
+			found_mapping = 1;
+			lcmaps_log_debug(3,"%s: found username: %s\n", logstr, username);
+		    }
+		    else if (strcmp(username, requested_username) == 0)
+		    {
+			found_mapping = 1;
+			lcmaps_log_debug(3,"%s: found requested username: %s\n", logstr, username);
+		    }
+		    else
+		    {
+			lcmaps_log(LOG_NOTICE,"%s: The requested username \"%s\" does not match the poolaccount \"%s\" for %s in %s, mapcount=%d\n",
+				logstr, requested_username, username, dn, gridmapfile, mapcounter);
+			goto fail_voms_poolaccount;
+		    }
                     break;
                 }
             }
