@@ -470,6 +470,7 @@ static int plugin_run_or_verify(
     gid_t *                 requested_pgid_list = NULL;
     int                     requested_nsgid     = 0;
     gid_t *                 requested_sgid_list = NULL;
+    void *                  value               = NULL;
 
     /*
      * The beginning
@@ -492,11 +493,14 @@ static int plugin_run_or_verify(
      */
     if (lcmaps_mode == PLUGIN_VERIFY)
     {
-        if ( (requested_npgid = *(int *) lcmaps_getArgValue("requested_npgid", "int", argc, argv)) > 0 )
+        if ( (value = lcmaps_getArgValue("requested_npgid", "int", argc, argv)) > 0 )
         {
+	    requested_npgid = *(int *) value;
             lcmaps_log_debug(1,"%s: the list of pgids should contain %d elements\n", logstr, requested_npgid);
-            if ( ( requested_pgid_list = *(gid_t **) lcmaps_getArgValue("requested_pgid_list", "gid_t *", argc, argv) ) )
+            if ( ( value = lcmaps_getArgValue("requested_pgid_list", "gid_t *", argc, argv) ) )   {
+		requested_pgid_list = *(gid_t **) value;
                 lcmaps_log_debug(1, "%s: found list of pgids\n", logstr);
+	    }
             else
             {
                 lcmaps_log_debug(1, "%s: could not retrieve list of pgids (failure)!\n", logstr);
@@ -507,11 +511,14 @@ static int plugin_run_or_verify(
                 lcmaps_log_debug(3, "%s: pgid[%d]: %d\n", logstr, i, (int)(requested_pgid_list[i]));
             }
         }
-        if ( (requested_nsgid = *(int *) lcmaps_getArgValue("requested_nsgid", "int", argc, argv)) > 0 )
+        if ( (value = lcmaps_getArgValue("requested_nsgid", "int", argc, argv)) > 0 )
         {
+	    requested_nsgid = *(int *) value;
             lcmaps_log_debug(1,"%s: the list of sgids should contain %d elements\n", logstr, requested_nsgid);
-            if ( ( requested_sgid_list = *(gid_t **) lcmaps_getArgValue("requested_sgid_list", "gid_t *", argc, argv) ) )
+            if ( ( value = lcmaps_getArgValue("requested_sgid_list", "gid_t *", argc, argv) ) )   {
+		requested_sgid_list = *(gid_t **) value;
                 lcmaps_log_debug(1, "%s: found list of sgids\n", logstr);
+	    }
             else
             {
                 lcmaps_log_debug(1, "%s: could not retrieve list of sgids (failure)!\n", logstr);
@@ -534,11 +541,14 @@ static int plugin_run_or_verify(
      * The VOMS info has to matched against the info in the gridmapfile
      */
     lcmaps_log_debug(5,"%s: First try to get the FQAN list from input credential repository ...\n", logstr);
-    if ( ( nfqan = *(int *) lcmaps_getArgValue("nfqan", "int", argc, argv) ) )
+    if ( ( value = lcmaps_getArgValue("nfqan", "int", argc, argv) ) )
     {
+	nfqan = *(int *) value;
         lcmaps_log_debug(5,"%s: the list of FQANs should contain %d elements\n", logstr, nfqan);
-        if ( ( fqan_list = *(char ***) lcmaps_getArgValue("fqan_list", "char **", argc, argv) ) )
+        if ( ( value = lcmaps_getArgValue("fqan_list", "char **", argc, argv) ) )   {
+	    fqan_list = *(char ***) value;
             lcmaps_log_debug(5, "%s: found list of FQANs\n", logstr);
+	}
         else
         {
             lcmaps_log(LOG_NOTICE, "%s: could not retrieve list of FQANs (failure)!\n", logstr);
